# Copyright Contributors to the DNF5 project.
# Copyright Contributors to the libdnf project.
# SPDX-License-Identifier: GPL-2.0-or-later
#
# This file is part of libdnf: https://github.com/rpm-software-management/libdnf/
#
# Libdnf is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Libdnf is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libdnf.  If not, see <https://www.gnu.org/licenses/>.

import libdnf5

import base_test_case


class TestAdvisory(base_test_case.BaseTestCase):
    def setUp(self):
        super().setUp()
        self.add_repo_repomd("repomd-repo1")
        query = libdnf5.advisory.AdvisoryQuery(self.base)
        query.filter_type("security")
        self.advisory = next(iter(query))

    def test_advisory_query(self):
        query = libdnf5.advisory.AdvisoryQuery(self.base)
        query.filter_name("DNF-2019-1")
        self.assertEqual(query.size(), 1)
        advisory = next(iter(query))
        self.assertEqual(advisory.get_name(), "DNF-2019-1")

        query = libdnf5.advisory.AdvisoryQuery(self.base)
        query.filter_type("bugfix")
        self.assertEqual(query.size(), 2)
        advisory = next(iter(query))
        self.assertEqual(advisory.get_type(), "bugfix")

        query = libdnf5.advisory.AdvisoryQuery(self.base)
        query.filter_reference("1111")
        self.assertEqual(query.size(), 1)
        advisory = next(iter(query))
        refs = advisory.get_references()
        self.assertEqual(len(refs), 1)
        self.assertEqual(refs[0].get_id(), "1111")

        query = libdnf5.advisory.AdvisoryQuery(self.base)
        query.filter_severity("moderate")
        self.assertEqual(query.size(), 1)
        advisory = next(iter(query))
        self.assertEqual(advisory.get_severity(), "moderate")

        query = libdnf5.advisory.AdvisoryQuery(self.base)
        pkgquery = libdnf5.rpm.PackageQuery(self.base)
        query.filter_packages(pkgquery)
        self.assertEqual(query.size(), 1)
        advisory = next(iter(query))
        self.assertEqual(advisory.get_name(), "DNF-2019-1")

        query = libdnf5.advisory.AdvisoryQuery(self.base)
        pkgquery = libdnf5.rpm.PackageQuery(self.base)
        pkgs = query.get_advisory_packages_sorted(pkgquery)
        self.assertEqual(pkgs.size(), 1)
        self.assertEqual(pkgs[0].get_nevra(), "pkg-1.2-3.x86_64")

    def test_advisory(self):
        self.assertEqual(self.advisory.get_name(), "DNF-2019-1")
        self.assertEqual(self.advisory.get_severity(), "moderate")
        self.assertEqual(self.advisory.get_type(), "security")
        self.assertEqual(self.advisory.get_buildtime(), 1550849401)
        self.assertEqual(self.advisory.get_vendor(), "dnf-testing@redhat.com")
        self.assertEqual(self.advisory.get_description(),
                         "testing advisory 2019")
        self.assertEqual(self.advisory.get_title(), "bugfix_A-1.0-1")
        self.assertEqual(self.advisory.get_status(), "stable")
        self.assertEqual(self.advisory.get_rights(), "")
        self.assertEqual(self.advisory.get_message(), "")
        self.assertEqual(self.advisory.is_applicable(), True)

    def test_references(self):
        refs = self.advisory.get_references()
        self.assertEqual(len(refs), 1)
        reference = refs[0]
        self.assertEqual(reference.get_id(), "1111")
        self.assertEqual(reference.get_type(), "cve")
        self.assertEqual(reference.get_title(), "CVE-2999")
        self.assertEqual(reference.get_url(), "https://foobar/foobarupdate_2")

    def test_collections(self):
        cols = self.advisory.get_collections()
        self.assertEqual(len(cols), 1)
        collection = cols[0]
        self.assertEqual(collection.get_advisory_id(), self.advisory.get_id())
        self.assertEqual(collection.get_advisory(), self.advisory)
        self.assertEqual(collection.is_applicable(), True)

    def test_advisory_packages(self):
        collection = self.advisory.get_collections()[0]
        adv_packages = collection.get_packages()
        self.assertEqual(len(adv_packages), 2)
        adv_package = adv_packages[0]

        self.assertEqual(adv_package.get_advisory_id(), self.advisory.get_id())
        self.assertEqual(adv_package.get_advisory(), self.advisory)
        # We can't compare the collections directly because adv_package constructs
        # a new instance of the same collection
        self.assertEqual(adv_package.get_advisory_collection(
        ).get_advisory_id(), self.advisory.get_id())

        self.assertEqual(adv_package.get_name(), "pkg")
        self.assertEqual(adv_package.get_epoch(), "0")
        self.assertEqual(adv_package.get_version(), "1.2")
        self.assertEqual(adv_package.get_release(), "3")
        self.assertEqual(adv_package.get_evr(), "1.2-3")
        self.assertEqual(adv_package.get_arch(), "x86_64")
        self.assertEqual(adv_package.get_nevra(), "pkg-1.2-3.x86_64")

        self.assertEqual(adv_package.get_reboot_suggested(), True)
        self.assertEqual(adv_package.get_restart_suggested(), False)
        self.assertEqual(adv_package.get_relogin_suggested(), False)

    def test_advisory_modules(self):
        collection = self.advisory.get_collections()[0]
        adv_modules = collection.get_modules()
        self.assertEqual(len(adv_modules), 2)
        adv_module = adv_modules[0]

        self.assertEqual(adv_module.get_advisory_id(), self.advisory.get_id())
        self.assertEqual(adv_module.get_advisory(), self.advisory)
        # We can't compare the collections directly because adv_module constructs
        # a new instance of the same collection
        self.assertEqual(adv_module.get_advisory_collection(
        ).get_advisory_id(), self.advisory.get_id())

        self.assertEqual(adv_module.get_name(), "perl-DBI")
        self.assertEqual(adv_module.get_stream(), "master")
        self.assertEqual(adv_module.get_version(), "2")
        self.assertEqual(adv_module.get_context(), "2a")
        self.assertEqual(adv_module.get_arch(), "x86_64")
        self.assertEqual(adv_module.get_nsvca(), "perl-DBI:master:2:2a:x86_64")



# Copyright Contributors to the DNF5 project.
# Copyright Contributors to the libdnf project.
# SPDX-License-Identifier: GPL-2.0-or-later
#
# This file is part of libdnf: https://github.com/rpm-software-management/libdnf/
#
# Libdnf is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Libdnf is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libdnf.  If not, see <https://www.gnu.org/licenses/>.

import libdnf5

import base_test_case


class TestConfigurationOptions(base_test_case.BaseTestCase):
    def test_set_with_runtime_priority(self):
        proxy = self.base.get_config().get_proxy_option()
        self.assertEqual(proxy.get_priority(),
                         libdnf5.conf.Option.Priority_DEFAULT)

        proxy.set('abcd')
        self.assertEqual(proxy.get_value(), 'abcd')
        self.assertEqual(proxy.get_priority(),
                         libdnf5.conf.Option.Priority_RUNTIME)

    def test_container_add_item(self):
        auths_config = self.base.get_config().get_proxy_auth_method_option()
        auths_config.set(('basic', 'ntlm'))
        auths_config.add_item(libdnf5.conf.Option.Priority_RUNTIME, 'digest')
        self.assertEqual(auths_config.get_value(), ('basic', 'digest', 'ntlm'))

    def test_container_add(self):
        types_config = self.base.get_config().get_optional_metadata_types_option()
        # optional_metadata_types is an append option with non-empty default.
        # To test anything, clear it first
        types_config.set(())
        types_config.set((libdnf5.conf.METADATA_TYPE_FILELISTS,))
        types_config.add(libdnf5.conf.Option.Priority_RUNTIME, (libdnf5.conf.METADATA_TYPE_COMPS,
                         libdnf5.conf.METADATA_TYPE_UPDATEINFO))
        self.assertEqual(types_config.get_value(), (libdnf5.conf.METADATA_TYPE_COMPS,
                         libdnf5.conf.METADATA_TYPE_FILELISTS, libdnf5.conf.METADATA_TYPE_UPDATEINFO))

    def test_set_by_attribute(self):
        config = self.base.get_config()
        config.comment = 'some comment'
        comment_option = config.get_comment_option()
        self.assertEqual(comment_option.get_value(), 'some comment')
        self.assertEqual(comment_option.get_priority(),
                         libdnf5.conf.Option.Priority_RUNTIME)

    def test_get_by_attribute(self):
        config = self.base.get_config()
        config.get_comment_option().set('new comment')
        self.assertEqual(config.comment, 'new comment')

    def test_set_get_by_attribute(self):
        config = self.base.get_config()
        config.comment = 'test'
        self.assertEqual(config.comment, 'test')

    def test_writing_to_locked_option(self):
        config = self.base.get_config()

        option = config.get_keepcache_option()
        option.lock('')

        with self.assertRaisesRegex(libdnf5.exception.UserAssertionError,
                                    'libdnf5/conf/option.cpp:[0-9]+: void libdnf5::Option::assert_not_locked\\(\\) const:'
                                    ' API Assertion \'!p_impl->locked\' failed:'
                                    ' Attempting to write to a locked option'):
            option.set(False)

    def test_get_unset_option_by_attribute(self):
        config = self.base.get_config()
        destdir = config.destdir
        self.assertEqual(destdir, None)

    def test_get_unknown_option_by_attribute(self):
        config = self.base.get_config()
        self.assertRaises(AttributeError, lambda: config.xyz)

    def test_iterate_options(self):
        config = self.base.get_config()

        config.proxy = 'abcd'

        proxy_option = None
        for option in config.opt_binds():
            if option.first == 'proxy':
                proxy_option = (option.first, option.second.get_value_string())
                break

        self.assertEqual(proxy_option, ('proxy', 'abcd'))



# Copyright Contributors to the DNF5 project.
# Copyright Contributors to the libdnf project.
# SPDX-License-Identifier: GPL-2.0-or-later
#
# This file is part of libdnf: https://github.com/rpm-software-management/libdnf/
#
# Libdnf is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Libdnf is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libdnf.  If not, see <https://www.gnu.org/licenses/>.

import dbus
import os

import support


class RepoTest(support.InstallrootCase):

    def test_repoquery_all(self):
        # get list of all available packages
        pkglist = self.iface_rpm.list(
            {"package_attrs": ["full_nevra", "repo_id"]})
        # id of package depends on order of the repos in the sack which varies
        # between runs so we can't rely on the value
        for pkg in pkglist:
            pkg.pop('id')
        # packages are ordered by the id hence assertCountEqual
        self.assertCountEqual(
            pkglist,
            dbus.Array([
                dbus.Dictionary({
                    dbus.String('full_nevra'): dbus.String('one-0:1-1.noarch', variant_level=1),
                    dbus.String('repo_id'): dbus.String('rpm-repo1', variant_level=1)},
                    signature=dbus.Signature('sv')),
                dbus.Dictionary({
                    dbus.String('full_nevra'): dbus.String('one-0:1-1.src', variant_level=1),
                    dbus.String('repo_id'): dbus.String('rpm-repo1', variant_level=1)},
                    signature=dbus.Signature('sv')),
                dbus.Dictionary({
                    dbus.String('full_nevra'): dbus.String('one-0:2-1.noarch', variant_level=1),
                    dbus.String('repo_id'): dbus.String('rpm-repo1', variant_level=1)},
                    signature=dbus.Signature('sv')),
                dbus.Dictionary({
                    dbus.String('full_nevra'): dbus.String('one-0:2-1.src', variant_level=1),
                    dbus.String('repo_id'): dbus.String('rpm-repo1', variant_level=1)},
                    signature=dbus.Signature('sv')),
                dbus.Dictionary({
                    dbus.String('full_nevra'): dbus.String('two-0:2-2.noarch', variant_level=1),
                    dbus.String('repo_id'): dbus.String('rpm-repo2', variant_level=1)},
                    signature=dbus.Signature('sv')),
                dbus.Dictionary({
                    dbus.String('full_nevra'): dbus.String('two-0:2-2.src', variant_level=1),
                    dbus.String('repo_id'): dbus.String('rpm-repo2', variant_level=1)},
                    signature=dbus.Signature('sv')),
            ],
                signature=dbus.Signature('a{sv}'))
        )

    def test_repoquery_match(self):
        # get list of all available packages
        pkglist = self.iface_rpm.list({
            "package_attrs": ["full_nevra", "repo_id"],
            "patterns": ["one"]})
        # id of package depends on order of the repos in the sack which varies
        # between runs so we can't rely on the value
        for pkg in pkglist:
            pkg.pop('id')
        # packages are ordered by the id hence assertCountEqual
        self.assertCountEqual(
            pkglist,
            dbus.Array([
                dbus.Dictionary({
                    dbus.String('full_nevra'): dbus.String('one-0:1-1.noarch', variant_level=1),
                    dbus.String('repo_id'): dbus.String('rpm-repo1', variant_level=1)},
                    signature=dbus.Signature('sv')),
                dbus.Dictionary({
                    dbus.String('full_nevra'): dbus.String('one-0:1-1.src', variant_level=1),
                    dbus.String('repo_id'): dbus.String('rpm-repo1', variant_level=1)},
                    signature=dbus.Signature('sv')),
                dbus.Dictionary({
                    dbus.String('full_nevra'): dbus.String('one-0:2-1.noarch', variant_level=1),
                    dbus.String('repo_id'): dbus.String('rpm-repo1', variant_level=1)},
                    signature=dbus.Signature('sv')),
                dbus.Dictionary({
                    dbus.String('full_nevra'): dbus.String('one-0:2-1.src', variant_level=1),
                    dbus.String('repo_id'): dbus.String('rpm-repo1', variant_level=1)},
                    signature=dbus.Signature('sv')),
            ],
                signature=dbus.Signature('a{sv}'))
        )



class TransactionCallbacks(libdnf5.rpm.TransactionCallbacks):
    def install_stop(self, item, amount, total):
        if item.get_action() == libdnf5.transaction.TransactionItemAction_INSTALL:
            print(f"Installed: {item.get_package().get_full_nevra()}")

    def script_start(self, item, nevra, type):
        script = libdnf5.rpm.TransactionCallbacks.script_type_to_string(type)
        nevra_str = libdnf5.rpm.to_full_nevra_string(nevra)
        print(f"Running {script} scriptlet for: {nevra_str}")

    def unpack_error(self, item):
        print(f"Unpack error: {item.get_package().get_full_nevra()}")

    def cpio_error(self, item):
        print(f"Cpio error: {item.get_package().get_full_nevra()}")

    def script_error(self, item, nevra, type, return_code):
        script = libdnf5.rpm.TransactionCallbacks.script_type_to_string(type)
        nevra_str = libdnf5.rpm.to_full_nevra_string(nevra)
        print(f"Error in {script} scriptlet: {nevra_str} - return code {return_code}")


# Set the transaction callbacks.
transaction_callbacks = TransactionCallbacks()
transaction.set_callbacks(
    libdnf5.rpm.TransactionCallbacksUniquePtr(transaction_callbacks))

# Run the transaction.
transaction.run()



# Copyright Contributors to the DNF5 project.
# Copyright Contributors to the libdnf project.
# SPDX-License-Identifier: GPL-2.0-or-later
#
# This file is part of libdnf: https://github.com/rpm-software-management/libdnf/
#
# Libdnf is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Libdnf is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libdnf.  If not, see <https://www.gnu.org/licenses/>.

import unittest
import gc

import libdnf5.base

import base_test_case


class TestPackageDownloader(base_test_case.BaseTestCase):
    def test_package_downloader(self):
        class PackageDownloadCallbacks(libdnf5.repo.DownloadCallbacks):
            test_instance = None

            def __init__(self):
                super(PackageDownloadCallbacks, self).__init__()
                self.user_cb_data_container = []  # Hold references to user_cb_data
                self.start_cnt = 0
                self.progress_cnt = 0
                self.mirror_failure_cnt = 0
                self.end_cnt = 0
                self.user_data_array = []
                self.user_cb_data_array = []
                self.end_status = []
                self.end_msg = []

            def add_new_download(self, user_data, description, total_to_download):
                self.start_cnt += 1
                self.user_data_array.append(user_data)
                user_cb_data_reference = "Package: " + description
                self.user_cb_data_container.append(user_cb_data_reference)
                return len(self.user_cb_data_container) - 1

            def end(self, user_cb_data, status, msg):
                self.end_cnt += 1
                self.test_instance.assertIn(user_cb_data, [0, 1])
                self.user_cb_data_array.append(user_cb_data)
                self.end_status.append(status)
                self.end_msg.append(msg)
                return 0

            def progress(self, user_cb_data, total_to_download, downloaded):
                self.progress_cnt += 1
                self.test_instance.assertIn(user_cb_data, [0, 1])
                return 0

            def mirror_failure(self, user_cb_data, msg, url):
                self.mirror_failure_cnt += 1
                self.test_instance.assertIn(user_cb_data, [0, 1])
                return 0

        PackageDownloadCallbacks.test_instance = self

        repo = self.add_repo_rpm("rpm-repo1")

        query = libdnf5.rpm.PackageQuery(self.base)
        query.filter_name(["one"])
        query.filter_arch(["noarch"])
        self.assertEqual(query.size(), 2)

        downloader = libdnf5.repo.PackageDownloader(self.base)

        cbs = PackageDownloadCallbacks()
        self.base.set_download_callbacks(
            libdnf5.repo.DownloadCallbacksUniquePtr(cbs))

        user_data = 2
        for package in query:
            downloader.add(package, user_data)
            user_data *= 5

        downloader.download()

        # forcefully deallocate the downloader, to check cbs is still valid
        downloader = None
        gc.collect()

        cbs.user_cb_data_container.sort()
        self.assertEqual(cbs.user_cb_data_container, [
                         "Package: one-0:1-1.noarch", "Package: one-0:2-1.noarch"])

        self.assertEqual(cbs.start_cnt, 2)
        self.assertGreaterEqual(cbs.progress_cnt, 2)
        self.assertEqual(cbs.mirror_failure_cnt, 0)
        self.assertEqual(cbs.end_cnt, 2)

        self.assertEqual(cbs.user_data_array, [2, 10])

        cbs.user_cb_data_array.sort()
        self.assertEqual(cbs.user_cb_data_array, [0, 1])
        self.assertEqual(
            cbs.end_status,
            [PackageDownloadCallbacks.TransferStatus_SUCCESSFUL, PackageDownloadCallbacks.TransferStatus_SUCCESSFUL])
        self.assertEqual(cbs.end_msg, [None, None])


class TestPackageDownloaderReturnNone(base_test_case.BaseTestCase):
    # Previously, add_new_download could only return None. This test adds a
    # check to ensure backwards-compatibility with that API.
    def test_package_downloader_return_none(self):
        class PackageDownloadCallbacks(libdnf5.repo.DownloadCallbacks):
            def __init__(self):
                super(PackageDownloadCallbacks, self).__init__()

            def add_new_download(self, user_data, description, total_to_download):
                return None

        repo = self.add_repo_rpm("rpm-repo1")

        query = libdnf5.rpm.PackageQuery(self.base)
        query.filter_name(["one"])
        query.filter_arch(["noarch"])
        self.assertEqual(query.size(), 2)

        downloader = libdnf5.repo.PackageDownloader(self.base)

        cbs = PackageDownloadCallbacks()
        self.base.set_download_callbacks(
            libdnf5.repo.DownloadCallbacksUniquePtr(cbs))

        for package in query:
            downloader.add(package)

        downloader.download()



# Copyright Contributors to the DNF5 project.
# Copyright Contributors to the libdnf project.
# SPDX-License-Identifier: GPL-2.0-or-later
#
# This file is part of libdnf: https://github.com/rpm-software-management/libdnf/
#
# Libdnf is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Libdnf is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libdnf.  If not, see <https://www.gnu.org/licenses/>.

import dbus

import support


class InstallTest(support.InstallrootCase):

    def test_install_package(self):
        # install a package
        self.iface_rpm.install(['one'], dbus.Dictionary({}, signature='sv'))

        resolved, result = self.iface_goal.resolve(
            dbus.Dictionary({}, signature='sv'))
        self.sanitize_transaction(resolved)

        self.assertEqual(result, 0)
        self.assertCountEqual(
            resolved,
            dbus.Array([
                dbus.Struct((
                    dbus.String('Package'),     # object type
                    dbus.String('Install'),     # action
                    dbus.String('User'),        # reason
                    dbus.Dictionary({           # transaction item attrs
                    }, signature=dbus.Signature('sv')),
                    dbus.Dictionary({           # package
                        dbus.String('full_nevra'): dbus.String('one-0:2-1.noarch', variant_level=1),
                        dbus.String('arch'): dbus.String('noarch', variant_level=1),
                        dbus.String('epoch'): dbus.String('0', variant_level=1),
                        dbus.String('evr'): dbus.String('2-1', variant_level=1),
                        dbus.String('name'): dbus.String('one', variant_level=1),
                        dbus.String('install_size'): dbus.UInt64(0, variant_level=1),
                        dbus.String('release'): dbus.String('1', variant_level=1),
                        dbus.String('repo_id'): dbus.String('rpm-repo1', variant_level=1),
                        dbus.String('version'): dbus.String('2', variant_level=1),
                        dbus.String('from_repo_id'): dbus.String('', variant_level=1),
                        dbus.String('reason'): dbus.String('None', variant_level=1),
                    }, signature=dbus.Signature('sv'))),
                    signature=None)
            ], signature=dbus.Signature('(ua{sv})'))
        )

        self.iface_goal.do_transaction(dbus.Dictionary({}, signature='sv'))

    def test_install_no_skip_unavailable(self):
        '''with skip_unavailable=False attempt to install nonexistent package returns error'''
        self.iface_rpm.install(['no_one'], dbus.Dictionary(
            {'skip_unavailable': False}, signature='sv'))
        resolved, result = self.iface_goal.resolve(
            dbus.Dictionary({}, signature='sv'))

        self.assertEqual(result, 2)
        self.assertCountEqual(
            resolved,
            dbus.Array([
            ], signature=dbus.Signature('(ua{sv})'))
        )

        errors = self.iface_goal.get_transaction_problems_string()
        self.assertEqual(errors, dbus.Array([
            dbus.String('No match for argument: no_one')], signature=dbus.Signature('s')))

    def test_install_skip_unavailable(self):
        '''
        with skip_unavailable=True attempt to install nonexistent package returns empty
        transaction
        '''
        self.iface_rpm.install(['no_one'], dbus.Dictionary(
            {'skip_unavailable': True}, signature='sv'))

        resolved, result = self.iface_goal.resolve(
            dbus.Dictionary({}, signature='sv'))

        self.assertEqual(result, 1)
        self.assertCountEqual(
            resolved,
            dbus.Array([
            ], signature=dbus.Signature('(ua{sv})'))
        )

        errors = self.iface_goal.get_transaction_problems_string()
        self.assertEqual(errors, dbus.Array([
            dbus.String('No match for argument: no_one')], signature=dbus.Signature('s')))

    def test_install_fromrepo(self):
        '''
        attempt to install package from repo that does not contain it return error
        '''
        self.iface_rpm.install(['one'], dbus.Dictionary(
            {'repo_ids': ['rpm-repo2']}, signature='sv'))
        resolved, result = self.iface_goal.resolve(
            dbus.Dictionary({}, signature='sv'))

        self.assertEqual(result, 2)
        self.assertCountEqual(
            resolved,
            dbus.Array([
            ], signature=dbus.Signature("(ua{sv})")),
        )

        errors = self.iface_goal.get_transaction_problems_string()
        self.assertEqual(errors, dbus.Array([
            dbus.String("No match for argument 'one' in repositories 'rpm-repo2'")],
            signature=dbus.Signature('s')))

    def test_install_file(self):
        '''
        Test installation using rpm file on the disk.
        '''
        self.iface_rpm.install([self.path_to_repo_rpm(
            "rpm-repo1", "one-1-1.noarch.rpm")], dbus.Dictionary({}, signature='sv'))

        resolved, result = self.iface_goal.resolve(
            dbus.Dictionary({}, signature='sv'))
        self.sanitize_transaction(resolved)

        self.assertEqual(result, 0)
        self.assertCountEqual(
            resolved,
            dbus.Array([
                dbus.Struct((
                    dbus.String('Package'),     # object type
                    dbus.String('Install'),     # action
                    dbus.String('User'),        # reason
                    dbus.Dictionary({           # transaction item attrs
                    }, signature=dbus.Signature('sv')),
                    dbus.Dictionary({           # package
                        dbus.String('full_nevra'): dbus.String('one-0:1-1.noarch', variant_level=1),
                        dbus.String('arch'): dbus.String('noarch', variant_level=1),
                        dbus.String('epoch'): dbus.String('0', variant_level=1),
                        dbus.String('evr'): dbus.String('1-1', variant_level=1),
                        dbus.String('name'): dbus.String('one', variant_level=1),
                        dbus.String('install_size'): dbus.UInt64(0, variant_level=1),
                        dbus.String('release'): dbus.String('1', variant_level=1),
                        dbus.String('repo_id'): dbus.String('@commandline', variant_level=1),
                        dbus.String('version'): dbus.String('1', variant_level=1),
                        dbus.String('from_repo_id'): dbus.String('', variant_level=1),
                        dbus.String('reason'): dbus.String('None', variant_level=1),
                    }, signature=dbus.Signature('sv'))),
                    signature=None)
            ], signature=dbus.Signature('(ua{sv})'))
        )

        self.iface_goal.do_transaction(dbus.Dictionary({}, signature='sv'))



# Copyright Contributors to the DNF5 project.
# Copyright Contributors to the libdnf project.
# SPDX-License-Identifier: GPL-2.0-or-later
#
# This file is part of libdnf: https://github.com/rpm-software-management/libdnf/
#
# Libdnf is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Libdnf is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libdnf.  If not, see <https://www.gnu.org/licenses/>.

import os

import libdnf5

import base_test_case


class TestRpmSignature(base_test_case.BaseTestCase):
    def setUp(self):
        super().setUp()
        self.add_repo_repomd("repomd-repo1")

    def test_checking_signature(self):
        # Test wrapper for check_package_signature
        query = libdnf5.rpm.PackageQuery(self.base)
        query.filter_name(["pkg"])
        package = next(iter(query))

        rpm_sign = libdnf5.rpm.RpmSignature(self.base)
        result = rpm_sign.check_package_signature(package)
        self.assertEqual(result, libdnf5.rpm.RpmSignature.CheckResult_SKIPPED)

    def test_key_files_vector_wrapper(self):
        # Test wrapper for std::vector<KeyInfo>
        key_path = os.path.join(base_test_case.PROJECT_SOURCE_DIR,
                                "test/data/keys", "key.pub")
        rpm_sign = libdnf5.rpm.RpmSignature(self.base)
        keys = rpm_sign.parse_key_file(key_path)
        _ = [key.get_key_id() for key in keys]



# Copyright Contributors to the DNF5 project.
# Copyright Contributors to the libdnf project.
# SPDX-License-Identifier: GPL-2.0-or-later
#
# This file is part of libdnf: https://github.com/rpm-software-management/libdnf/
#
# Libdnf is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Libdnf is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libdnf.  If not, see <https://www.gnu.org/licenses/>.


import unittest

import libdnf5

import base_test_case


class TestGroup(base_test_case.BaseTestCase):
    def test_group(self):
        self.add_repo_repomd("repomd-comps-core")
        q_core = libdnf5.comps.GroupQuery(self.base)
        core = q_core.get()

    def test_group_query_without_setup(self):
        # Create a new Base object
        base = libdnf5.base.Base()

        # Try to create a group query without running base.setup()
        with self.assertRaisesRegex(libdnf5.exception.UserAssertionError,
                                    'base_impl.hpp:[0-9]+: libdnf5::solv::CompsPool& libdnf5::Base::Impl::get_comps_pool\\(\\):'
                                    ' API Assertion \'comps_pool\' failed:'
                                    ' Base instance was not fully initialized by Base::setup\\(\\)'):
            libdnf5.comps.GroupQuery(base)

    def test_group_get_packages(self):
        self.add_repo_repomd("repomd-comps-core")
        query = libdnf5.comps.GroupQuery(self.base)
        core_group = next(iter(query))
        self.assertEqual(5, len(core_group.get_packages()))

    def test_group_when_query_out_of_scope(self):
        self.add_repo_repomd("repomd-comps-core")

        def get_group(base):
            query = libdnf5.comps.GroupQuery(base)
            return next(iter(query))

        self.assertEqual(5, len(get_group(self.base).get_packages()))

    def test_iterating_over_inline_query(self):
        self.add_repo_repomd("repomd-comps-core", False)
        self.add_repo_repomd("repomd-comps-standard")

        ids = []
        for grp in sorted(libdnf5.comps.GroupQuery(self.base), key=lambda x: int(x.get_order_int())):
            ids.append(grp.get_groupid())

        self.assertEqual(['core', 'standard'], ids)



# Copyright Contributors to the DNF5 project.
# Copyright Contributors to the libdnf project.
# SPDX-License-Identifier: GPL-2.0-or-later
#
# This file is part of libdnf: https://github.com/rpm-software-management/libdnf/
#
# Libdnf is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Libdnf is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libdnf.  If not, see <https://www.gnu.org/licenses/>.

import unittest

import libdnf5

import base_test_case


class TestRepoQuery(base_test_case.BaseTestCase):
    def test_repo_query(self):
        # Creates new repositories in the repo_sack
        repo1 = self.repo_sack.create_repo("repo1")
        repo1.enable()
        repo1.get_config().baseurl = "file:///path/to/repo1"

        repo2 = self.repo_sack.create_repo("repo2")
        repo2.disable()
        repo2.get_config().baseurl = "https://host/path/to/repo2"

        repo1_updates = self.repo_sack.create_repo("repo1_updates")
        repo1_updates.disable()
        repo1_updates.get_config().baseurl = "https://host/path/to/repo1_updates"

        repo2_updates = self.repo_sack.create_repo("repo2_updates")
        repo2_updates.enable()
        repo2_updates.get_config().baseurl = "https://host/path/to/repo2_updates"

        # create a RepoQuery and test that it contains expected repos
        repo_query = libdnf5.repo.RepoQuery(self.base)
        self.assertEqual(repo_query.size(), 4)
        self.assertEqual(len(repo_query), 4)
        self.assertEqual(set(repo_query), {
                         repo1, repo2, repo1_updates, repo2_updates})

        # Tests filter_enabled method
        repo_query.filter_enabled(True)
        self.assertEqual(set(repo_query), {repo1, repo2_updates})

        # Tests filter_id method
        repo_query1 = libdnf5.repo.RepoQuery(self.base)
        repo_query1.filter_id("*updates", libdnf5.common.QueryCmp_GLOB)
        self.assertEqual(set(repo_query1), {repo1_updates, repo2_updates})

        # Tests filter_local method
        repo_query2 = libdnf5.repo.RepoQuery(self.base)
        repo_query2.filter_local(False)
        self.assertEqual(set(repo_query2), {
                         repo2, repo1_updates, repo2_updates})

        # Tests iteration over RepoQuery object
        repo_query3 = libdnf5.repo.RepoQuery(self.base)
        result = set()
        for repo in repo_query3:
            result.add(repo.get_id())
        self.assertEqual(
            result, {"repo1", "repo2", "repo1_updates", "repo2_updates"})

        # Tests repo objects after query goes out of scope
        repos_list = []
        for repo in libdnf5.repo.RepoQuery(self.base):
            repos_list.append(repo)
        result = set()
        for repo in repos_list:
            result.add(repo.get_id())
        self.assertEqual(
            result, {"repo1", "repo2", "repo1_updates", "repo2_updates"})



repo_sack = base.get_repo_sack()

# Optionally, read repositories from system configuration files.
repo_sack.create_repos_from_system_configuration()

# Optionally, create and configure a new repository.
repo = repo_sack.create_repo("my_new_repo_id")
repo.get_config().baseurl = baseurl

# Load repositories. To limit which repositories are loaded, pass
# a repository type (e.g. libdnf5.repo.Repo.Type_SYSTEM).
repo_sack.load_repos()
